require "import"
import "android.widget.*"
import "android.content.Context"
import "android.net.Uri"
import "android.content.Intent"
import "android.view.KeyEvent"
import "com.androlua.*"
import "android.os.Vibrator"

-- Crear contexto
local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(100)

-- Función para abrir enlaces
function openLink(url)
    local intent = Intent(Intent.ACTION_VIEW)
    intent.setData(Uri.parse(url))
    context.startActivity(intent)
    service.asyncSpeak("El enlace se ha abierto.")
    dlg.dismiss()  -- Cerrar el diálogo al abrir el enlace
end

-- Función para reproducir sonido
function playSound()
    service.playSoundTick()  -- Reproducir sonido
end

-- Función para convertir texto a una operación matemática
function convertToOperation(text)
    -- Reemplazar palabras por sus equivalentes matemáticos
    text = text:gsub("cuánto es", ""):gsub("¿", ""):gsub("?", ""):gsub("más", "+"):gsub("menos", "-"):gsub("por", "*"):gsub("entre", "/"):gsub("uno", "1"):gsub("dos", "2"):gsub("tres", "3"):gsub("cuatro", "4"):gsub("cinco", "5"):gsub("seis", "6"):gsub("siete", "7"):gsub("ocho", "8"):gsub("nueve", "9"):gsub("cero", "0")
    
    -- Manejo de raíces cuadradas y otras funciones
    text = text:gsub("raíz cuadrada de", "math.sqrt"):gsub("raíz de", "math.sqrt"):gsub("potencia de", "^"):gsub("elevado a", "^")  -- Aquí puedes añadir más funciones si lo deseas
    return text
end

-- Layout principal de la calculadora
local calculatorLayout = {
    LinearLayout,
    orientation = LinearLayout.VERTICAL,
    {
        TextView,
        text = "Bienvenido a la Calculadora Accesible",
        textSize = "20sp",
        gravity = 17,
        padding = "16dp",
    },
    {
        EditText,
        id = "operationInput",
        hint = "Escribe la operación aquí...",
        layout_width = "match_parent",
        layout_height = "wrap_content",
        padding = "16dp",
    },
    {
        Button,
        text = "Calcular",
        onClick = function(view)
            playSound()  -- Reproducir sonido
            local operation = operationInput.Text  -- Obtener texto del EditText
            -- Convertir la entrada a una operación
            operation = convertToOperation(operation)

            -- Evaluar la operación
            local success, result = pcall(function() return load("return " .. operation)() end)

            if success then
                -- Mostrar y hablar el resultado
                service.asyncSpeak("El resultado es: " .. tostring(result))
                resultView.Text = "Resultado: " .. tostring(result)
            else
                service.asyncSpeak("Error en la operación.")
                resultView.Text = "Error en la operación."
            end
        end,
        layout_width = "wrap_content",
        gravity = 5,
    },
    {
        TextView,
        id = "resultView",
        text = "Resultado: ",
        layout_width = "wrap_content",
        layout_height = "wrap_content",
        padding = "16dp",
    },
    {
        Button,
        text = "Dictado por voz",
        onClick = function(view)
            playSound()  -- Reproducir sonido
            if service.execute("Dictado por voz") then
                return true
            end
        end,
        layout_width = "wrap_content",
        gravity = 5,
    },
    {
        Button,
        text = "Acerca de",
        onClick = function(view)
            playSound()  -- Reproducir sonido
            showAboutLayout()  -- Mostrar la pantalla de Acerca de
        end,
        layout_width = "wrap_content",
        gravity = 5,
    },
    {
        Button,
        text = "Cerrar",
        onClick = function(view)
            dlg.dismiss()  -- Cerrar el diálogo
        end,
        layout_width = "wrap_content",
        gravity = 5,
    }
}

-- Función para mostrar el layout de Acerca de
function showAboutLayout()
    dlg.dismiss()  -- Cerrar el diálogo actual
    local aboutLayout = {
        LinearLayout,
        orientation = LinearLayout.VERTICAL,
        {
            TextView,
            text = "¡Apóyanos haciendo clic en los enlaces a continuación! Cada clic nos ayuda a seguir desarrollando más plugins y mejorar la accesibilidad.",
            textSize = "16sp",
            gravity = 17,
            padding = "16dp",
        },
        {
            Button,
            text = "Canal Primario",
            onClick = function(view)
                playSound()  -- Reproducir sonido
                openLink("https://youtube.com/@cosasaccesibles1234?si=cBR1yld78Yg9qtcd")  -- Abrir enlace
            end,
            layout_width = "wrap_content",
            gravity = 5,
        },
        {
            Button,
            text = "Canal Secundario",
            onClick = function(view)
                playSound()  -- Reproducir sonido
                openLink("https://youtube.com/@historiasparanormales51?si=BbqZCyqe4E5l_Ych")  -- Abrir enlace
            end,
            layout_width = "wrap_content",
            gravity = 5,
        },
        {
            Button,
            text = "Sígueme en TikTok",
            onClick = function(view)
                playSound()  -- Reproducir sonido
                openLink("https://www.tiktok.com/@cristianmurguia54?_t=8iZMbOXmpEU&_r=1")  -- Abrir enlace
            end,
            layout_width = "wrap_content",
            gravity = 5,
        },
        {
            Button,
            text = "Canal de WhatsApp",
            onClick = function(view)
                playSound()  -- Reproducir sonido
                openLink("https://whatsapp.com/channel/0029Va5iI5N7dmeRittxuN05")  -- Abrir enlace
            end,
            layout_width = "wrap_content",
            gravity = 5,
        },
        {
            Button,
            text = "Navegar hacia Arriba",
            onClick = function(view)
                playSound()  -- Reproducir sonido
                dlg.dismiss()  -- Cerrar el diálogo de Acerca de
                showMainLayout()  -- Regresar a la pantalla principal
            end,
            layout_width = "wrap_content",
            gravity = 5,
        },
    }

    dlg = LuaDialog(context)
    dlg.View = loadlayout(aboutLayout)
    dlg.show()
end

-- Función para mostrar el layout principal de la calculadora
function showMainLayout()
    playSound()  -- Reproducir sonido al abrir el plugin
    dlg = LuaDialog(context)
    dlg.View = loadlayout(calculatorLayout)
    dlg.show()
end

-- Iniciar el plugin mostrando el layout principal
showMainLayout()

-- Manejar el evento de la tecla de retroceso
function onKeyDown(code, event)
    if code == KeyEvent.KEYCODE_BACK then
        if dlg then
            dlg.dismiss()
        end
        return true
    end
end

return true